import { extname, relative, resolve } from 'path';
import { fileURLToPath } from 'node:url';
import { glob } from 'glob';
import { defineConfig } from 'vite';
import dts from 'vite-plugin-dts';
import vue from '@vitejs/plugin-vue';
import { libInjectCss } from 'vite-plugin-lib-inject-css';
import { peerDependencies } from './package.json';

export default defineConfig({
  resolve: {
    alias: {
      '@': resolve(__dirname, '/src'),
    },
  },
  build: {
    emptyOutDir: true,
    cssMinify: false,
    minify: false,
    lib: {
      entry: resolve(__dirname, 'src/index.ts'),
      formats: ['es'],
    },
    rollupOptions: {
      external: [
        ...Object.keys(peerDependencies),
      ],
      input: getInputEntries(),
      output: {
        assetFileNames: 'styles/index[extname]',
        entryFileNames: '[name].js',
      },
    },
  },
  plugins: [
    vue(),
    dts({
      include: ['src/**/*.ts', 'src/**/*.vue'],
      exclude: ['src/stories/**/*', 'src/**/*.stories.ts', 'src/**/*.stories.js', 'src/**/*.stories.vue'],
    }),
    {
      name: 'merge-css',
      enforce: 'post',
      apply: 'build',
      generateBundle(_, bundle) {
        let combinedCss = '';
        // 收集所有css内容
        for (const filename in bundle) {
          if (filename.endsWith('.css')) {
            const chunk = bundle[filename];
            if (chunk.type === 'asset') {
              combinedCss += chunk.source;
              delete bundle[filename];
            }
          }
        }

        // 添加合并后的css文件
        this.emitFile({
          type: 'asset',
          fileName: 'styles/index.css',
          source: combinedCss
        });

        // 手动添加css引用到入口文件
        const entryChunk = bundle['index.js'];
        if (entryChunk?.type === 'chunk') {
          entryChunk.code = `import './styles/index.css';\n${entryChunk.code}`;
        }
      }
    },
  ],
  css: {
    preprocessorOptions: {
      scss: {
        api: 'modern-compiler',
      },
    },
  },
});

function getInputEntries() {
  const inputEntries = Object.fromEntries(
    glob.sync('src/**/*.{ts,vue}', {
      ignore: [
        'src/stories/**/*',
        'src/**/*.stories.{ts,js,vue}',
        'src/**/*.story.{ts,js,vue}',
        'src/**/*.test.{ts,js}',
        'src/**/*.spec.{ts,js}',
        'src/**/__tests__/**/*'
      ]
    }).map(file => [
      relative(
        'src',
        file.slice(0, file.length - extname(file).length),
      ),
      fileURLToPath(new URL(file, import.meta.url)),
    ]),
  );
  return inputEntries;
}
